"""Validation workflow for switch subscription objects."""

from typing import Any

from orchestrator.targets import Target
from orchestrator.workflow import StepList, begin, done, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products.product_types.switch import Switch
from gso.services.lso_client import LSOState, anonymous_lso_interaction
from gso.services.netbox_client import NetboxClient


@step("Validate switch in Netbox")
def check_netbox_device(subscription: Switch) -> None:
    """Fetch the device in Netbox. Will raise an exception if it is not found."""
    NetboxClient().get_device_by_name(subscription.switch.fqdn)


@step("Check base config for drift")
def verify_base_config(subscription: dict[str, Any]) -> LSOState:
    """Workflow step for running a playbook that checks whether base config has drifted."""
    return {
        "playbook_name": "switch_base_config.yaml",
        "inventory": {"all": {"hosts": {subscription["switch"]["fqdn"]: None}}},
        "extra_vars": {
            "subscription_json": subscription,
            "verb": "deploy",
            "dry_run": "true",
            "is_verification_workflow": "true",
        },
    }


@workflow(
    "Validate switch subscription", target=Target.SYSTEM, initial_input_form=(wrap_modify_initial_input_form(None))
)
def validate_switch() -> StepList:
    """Validate an existing, active switch subscription."""
    return (
        begin
        >> store_process_subscription(Target.SYSTEM)
        >> unsync
        >> check_netbox_device
        >> anonymous_lso_interaction(verify_base_config)
        >> resync
        >> done
    )
